
TEXT = 'CharField'
BOOLEAN = 'BooleanField'
INTEGER = 'IntegerField'
FLOAT = 'FloatField'
DATE_TIME = 'DateTimeField'
TIMESTAMP = 'TimeStampField'
FOREIGN_KEY = 'ForeignKey'
JSON = 'JSONField'
GEOMETRY = 'GeometryField'
GEOGRAPHY = 'GeographyField'
